@@ -33,6 +33,7 @@ from grapheekdb.lib.exceptions import GrapheekNoSuchTraversalException
 from grapheekdb.lib.exceptions import GrapheekMissingKeyException
 from grapheekdb.lib.exceptions import GrapheekInvalidDataTypeException
 from grapheekdb.lib.exceptions import GrapheekUnknownScriptException
+from grapheekdb.lib.exceptions import GrapheekUnknownAlias
 
 from grapheekdb.backends.data.misc import build_attr_filter_funcs
 from grapheekdb.backends.data.misc import _get_exact_filters
@@ -171,6 +172,86 @@ class DotExport(object):
         return '\n'.join(lines)
 
 
+class EntityAggregate(object):
+
+    def __init__(self, graph, entity_kind, entity_iterator, pair_iterator=None, agg='+', asc=False):
+        self._graph = graph
+        self._entity_kind = entity_kind
+        self._entity_iterator = entity_iterator
+        self._pair_iterator = pair_iterator
+        self._agg = agg
+        self._asc = asc
+        # -------------------------------
+        self._current_iteration = None
+
+    def _get_results(self):
+        # Special case : if self._agg is None, then it means that current EntityAggregate
+        # has been created by another EntityAggregate (for instance, by method limit)
+        # so we'll just iterate on _pair_iterator
+        if self._agg is None:
+            return list(self._pair_iterator)
+        assert(len(self._agg) > 0)
+        agg_type = self._agg[0]
+        assert(agg_type in ['+', '%'])
+        field_name = self._agg[1:]
+        mult = 1 if self._asc else -1
+        if self._agg == '+':
+            counters = defaultdict(int)
+        else:
+            counters = defaultdict(float)
+        total = 0
+        entity_cache = {}
+        for entity_id in self._entity_iterator._iterate(on_item=False):
+            if field_name == '':
+                value = 1
+            else:
+                entity = entity_cache.get(entity_id, self._graph._item_from_id(self._entity_kind, entity_id))
+                entity_cache[entity_id] = entity
+                value = getattr(entity, field_name, 0)
+            counters[entity_id] += value
+            total += value
+        if total and (agg_type == '%'):
+            for entity_id, value in counters.items():
+                counters[entity_id] = float(counters[entity_id]) / float(total)  # This may raise an exception, it's up to the user to handle it...
+        ordered_counters = sorted(counters.items(), key=lambda t: t[1] * mult)
+        results = []
+        for entity_id, count in ordered_counters:
+            entity = entity_cache.get(entity_id, None)
+            if entity is None:
+                entity = self._graph._item_from_id(self._entity_kind, entity_id)
+            results.append((entity, count))
+        return results
+
+    def __iter__(self):
+        return self
+
+    def next(self):
+        if self._current_iteration is None:
+            self._current_iteration = iter(self._get_results())
+        try:
+            return self._current_iteration.next()
+        except StopIteration:
+            self._current_iteration = None
+        raise StopIteration
+
+    def entities(self):
+        def entity_iterator(iterator):
+            for entity, _ in iterator:
+                yield entity.get_id()
+        return EntityIterator(self._graph, self._entity_kind, {}, entity_iterator(self), parent=self._entity_iterator)
+
+    def limit(self, number):
+        def limit_iterator(iterator, number):
+            counter = 0
+            for entity_id, count in iterator:
+                if counter < number:
+                    yield (entity_id, count)
+                else:
+                    break  # This line is REALLY important (it stops outer iteration)
+                counter += 1
+        return EntityAggregate(self._graph, self._entity_kind, self._entity_iterator, limit_iterator(self, number), agg=None, asc=None)
+
+
 class EntityIterator(object):
 
     def __init__(self, graph, src_kind, filters, id_iterator, parent=None, neighbors_cache=None, context=None):
@@ -249,21 +330,21 @@ class EntityIterator(object):
     def bothV(self, *args, **filters):
         return self._jump(KIND_VERTEX, BOTH_VERTICES_SUFFIX, *args, **filters)
 
-    def IN(self, *args, **filters):
+    def in_(self, *args, **filters):
         assert(self._src_kind == KIND_VERTEX)
         count = check_and_get_count(args)
         if count == 1:
             return self.inE(**filters).inV()
-        return self.IN(count - 1, **filters).inE(*args, **filters).inV()
+        return self.in_(count - 1, **filters).inE(*args, **filters).inV()
 
-    def OUT(self, *args, **filters):
+    def out_(self, *args, **filters):
         assert(self._src_kind == KIND_VERTEX)
         count = check_and_get_count(args)
         if count == 1:
             return self.outE(**filters).outV()
-        return self.OUT(count - 1, **filters).outE(*args, **filters).outV()
+        return self.out_(count - 1, **filters).outE(*args, **filters).outV()
 
-    def BOTH(self, *args, **filters):
+    def both_(self, *args, **filters):
         assert(self._src_kind == KIND_VERTEX)
         count = check_and_get_count(args)
 
@@ -292,7 +373,7 @@ class EntityIterator(object):
                 parent=self,
                 neighbors_cache=self._neighbors_cache
             )
-        entity_iterator = self.BOTH(count - 1, **filters)
+        entity_iterator = self.both_(count - 1, **filters)
         return EntityIterator(
             self._graph,
             KIND_VERTEX,
@@ -324,15 +405,39 @@ class EntityIterator(object):
             raise GrapheekNoSuchTraversalException
         return self._jump(KIND_EDGE, BOTH_EDGES_SUFFIX, **filters)
 
-    def dedup(self):
-        def dedup_iterator(entity_iterator):
-            known = set()
-            for entity_id in entity_iterator:
+    def dedup(self, *aliases):
+        def dedup_iterator(entity_iterator, aliases):
+            if aliases:
+                alias_known = defaultdict(set)
+            else:
+                known = set()
+            for entity_id in entity_iterator._iterate():
+                if aliases:
+                    keys = []
+                    for alias in aliases:
+                        ctx_entity = entity_iterator._context.get(alias, None)
+                        if ctx_entity is None:
+                            raise GrapheekUnknownAlias("'%s' alias is not defined" % (alias,))
+                        keys.append((ctx_entity._kind, ctx_entity.get_id()))
+                    known = alias_known[tuple(keys)]
                 if entity_id in known:
                     continue
                 known.add(entity_id)
                 yield entity_id
-        return EntityIterator(self._graph, self._src_kind, {}, dedup_iterator(self._iterate()), parent=self)
+        return EntityIterator(self._graph, self._src_kind, {}, dedup_iterator(self, aliases), parent=self)
+
+    def without(self, *aliases):
+        def without_iterator(entity_iterator, aliases):
+            for entity_id in entity_iterator._iterate():  # Iterating updates the context ...
+                member = False
+                for alias in aliases:
+                    excluded_entity = entity_iterator._context.get(alias, UNDEFINED)
+                    if excluded_entity._kind == self._src_kind and excluded_entity.get_id() == entity_id:
+                        member = True
+                        break
+                if not member:
+                    yield entity_id
+        return EntityIterator(self._graph, self._src_kind, {}, without_iterator(self, aliases), parent=self)
 
     def idata(self):
         for item in self:
@@ -380,25 +485,14 @@ class EntityIterator(object):
             self._graph._transaction_rollback(txn)
             raise GrapheekDataException(repr(e))
 
-    def collect(self, *args):
+    def collect(self, *aliases):
         result = []
-        for entity in self:
-            result.append([self._context.get(alias, None) for alias in args])
+        for entity in self:  # Iterating updates the context ...
+            result.append([self._context.get(alias, None) for alias in aliases])
         return result
 
-    def agg(self, aggtype='+', asc=False):
-        assert(aggtype in ['+', '%'])
-        mult = 1 if asc else -1
-        counters = defaultdict(int)
-        total = 0
-        for entity_id in self._iterate():
-            counters[entity_id] += 1
-            total += 1
-        if total and (aggtype == '%'):
-            for entity_id, value in counters.items():
-                counters[entity_id] = float(counters[entity_id]) / float(total)
-        ordered_counters = sorted(counters.items(), key=lambda t: t[1] * mult)
-        return [(self._graph._item_from_id(self._src_kind, entity_id), count) for entity_id, count in ordered_counters]
+    def agg(self, agg='+', asc=False):
+        return EntityAggregate(self._graph, self._src_kind, self, pair_iterator=None, agg=agg, asc=asc)
 
     def call(self, _fname, *aliases, **params):
         for entity_id in self._iterate():  # Iterating updates the context ...
@@ -615,23 +709,23 @@ class Node(Entity):
         graph._remove(txn, build_key(KIND_VERTEX, target_id, IN_VERTICES_SUFFIX, source_id))
         graph._remove(txn, build_key(KIND_VERTEX, target_id, BOTH_VERTICES_SUFFIX, source_id))
 
-    def IN(self, *args, **filters):
+    def in_(self, *args, **filters):
         def self_iterator(entity_id):
             yield entity_id
         entity_iterator = EntityIterator(self._graph, KIND_VERTEX, {}, self_iterator(self.get_id()), context={})
-        return entity_iterator.IN(*args, **filters)
+        return entity_iterator.in_(*args, **filters)
 
-    def OUT(self, *args, **filters):
+    def out_(self, *args, **filters):
         def self_iterator(entity_id):
             yield entity_id
         entity_iterator = EntityIterator(self._graph, KIND_VERTEX, {}, self_iterator(self.get_id()), context={})
-        return entity_iterator.OUT(*args, **filters)
+        return entity_iterator.out_(*args, **filters)
 
-    def BOTH(self, *args, **filters):
+    def both_(self, *args, **filters):
         def self_iterator(entity_id):
             yield entity_id
         entity_iterator = EntityIterator(self._graph, KIND_VERTEX, {}, self_iterator(self.get_id()), context={})
-        return entity_iterator.BOTH(*args, **filters)
+        return entity_iterator.both_(*args, **filters)
 
     def inE(self, **filters):
         return self._jump(KIND_EDGE, IN_EDGES_SUFFIX, **filters)
@@ -694,6 +788,14 @@ class Edge(Entity):
 
 class BaseGraph(GraphReadWrite):
 
+    def __del__(self):
+        # ensure connection is closed
+        try:
+            if not self._closed:
+                self._db_close()
+        except AttributeError:
+            pass
+
     def _prepare_database(self, txn):
         self._set(txn, PREPARED, 1)
         self._set(txn, METADATA_EDGE_COUNTER, 0)
@@ -713,6 +815,9 @@ class BaseGraph(GraphReadWrite):
 
     # Following methods MUST be overriden by child classes :
 
+    def _db_close(self):
+        raise NotImplementedError
+
     def _transaction_begin(self):
         raise NotImplementedError
 
@@ -1304,4 +1409,3 @@ class BaseGraph(GraphReadWrite):
                 class_module = sys.modules[module_path]
                 self._script_module_cache.append(class_module)
         self._script_cache = {}
-
