@@ -32,10 +32,13 @@ class KyotoCabinetGraph(BaseGraph):
             raise GrapheekDataKyotoCabinetInitFailureException(str(self._db.error()))
         super(KyotoCabinetGraph, self).__init__()
         self._ensure_prepared()
+        self._closed = False
 
-    def __del__(self):
-        # ensure connection is closed
-        self._db.close()
+    # Start method overriding :
+
+    def _db_close(self):
+        if not self._closed:
+            self._db.close()
 
     def _transaction_begin(self):
         self._db.begin_transaction()
@@ -88,12 +91,13 @@ class KyotoCabinetGraph(BaseGraph):
 
     def _bulk_get_lst(self, txn, keys):
         dic_values = self._db.get_bulk(keys)
-        if dic_values == None:
-            return []
         results = []
         for key in keys:
-            values = dic_values.get(key)
-            results.append(map(int, values.split('|')[1:]))
+            values = dic_values.get(key, UNDEFINED)
+            if values == UNDEFINED:
+                results.append([])
+            else:
+                results.append(map(int, values.split('|')[1:]))
         return results
 
     def _count_lst(self, txn, key):
@@ -111,4 +115,3 @@ class KyotoCabinetGraph(BaseGraph):
         # cause current entity and neighbour node can be linked multiple time
         new = old.replace('|%s' % value, '', 1)
         self._db.set(key, new)
-
