@@ -11,6 +11,12 @@ class LocalMemoryGraph(BaseGraph):
         self._dic = {}
         super(LocalMemoryGraph, self).__init__()
         self._ensure_prepared()
+        self._closed = False
+
+    # Start method overriding :
+
+    def _db_close(self):
+        pass
 
     def _transaction_begin(self):
         pass
